@@ -27,13 +27,6 @@
 
 #include "connman.h"
 
-static connman_bool_t global_offlinemode = FALSE;
-
-connman_bool_t __connman_manager_get_offlinemode(void)
-{
-	return global_offlinemode;
-}
-
 static void append_profiles(DBusMessageIter *dict)
 {
 	DBusMessageIter entry, value, iter;
@@ -207,6 +200,7 @@ static DBusMessage *get_properties(DBusConnection *conn,
 {
 	DBusMessage *reply;
 	DBusMessageIter array, dict;
+	connman_bool_t offlinemode;
 	const char *str;
 
 	DBG("conn %p", conn);
@@ -245,8 +239,9 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	connman_dbus_dict_append_variant(&dict, "State",
 						DBUS_TYPE_STRING, &str);
 
+	offlinemode = __connman_profile_get_offlinemode();
 	connman_dbus_dict_append_variant(&dict, "OfflineMode",
-				DBUS_TYPE_BOOLEAN, &global_offlinemode);
+					DBUS_TYPE_BOOLEAN, &offlinemode);
 
 	append_available_technologies(&dict);
 	append_enabled_technologies(&dict);
@@ -267,6 +262,7 @@ static DBusMessage *set_property(DBusConnection *conn,
 {
 	DBusMessageIter iter, value;
 	const char *name;
+	int type;
 
 	DBG("conn %p", conn);
 
@@ -281,19 +277,19 @@ static DBusMessage *set_property(DBusConnection *conn,
 					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
 		return __connman_error_permission_denied(msg);
 
+	type = dbus_message_iter_get_arg_type(&value);
+
 	if (g_str_equal(name, "OfflineMode") == TRUE) {
 		connman_bool_t offlinemode;
 
+		if (type != DBUS_TYPE_BOOLEAN)
+			return __connman_error_invalid_arguments(msg);
+
 		dbus_message_iter_get_basic(&value, &offlinemode);
 
-		if (global_offlinemode == offlinemode)
-			return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+		__connman_profile_set_offlinemode(offlinemode);
 
-		global_offlinemode = offlinemode;
-
-		__connman_storage_save_global();
-
-		__connman_device_set_offlinemode(offlinemode);
+		__connman_profile_save_default();
 	} else if (g_str_equal(name, "ActiveProfile") == TRUE) {
 		const char *str;
 
@@ -326,36 +322,55 @@ static DBusMessage *get_state(DBusConnection *conn,
 						DBUS_TYPE_INVALID);
 }
 
-static DBusMessage *add_profile(DBusConnection *conn,
+static DBusMessage *create_profile(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
-	const char *name;
+	const char *name, *path;
+	int err;
 
 	DBG("conn %p", conn);
 
 	dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &name,
 							DBUS_TYPE_INVALID);
 
-	return __connman_error_not_supported(msg);
+	if (__connman_security_check_privilege(msg,
+					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
+		return __connman_error_permission_denied(msg);
+
+	err = __connman_profile_create(name, &path);
+	if (err < 0)
+		return __connman_error_failed(msg, -err);
+
+	return g_dbus_create_reply(msg, DBUS_TYPE_OBJECT_PATH, &path,
+							DBUS_TYPE_INVALID);
 }
 
 static DBusMessage *remove_profile(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
 	const char *path;
+	int err;
 
 	DBG("conn %p", conn);
 
 	dbus_message_get_args(msg, NULL, DBUS_TYPE_OBJECT_PATH, &path,
 							DBUS_TYPE_INVALID);
 
-	return __connman_error_not_supported(msg);
+	if (__connman_security_check_privilege(msg,
+					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
+		return __connman_error_permission_denied(msg);
+
+	err = __connman_profile_remove(path);
+	if (err < 0)
+		return __connman_error_failed(msg, -err);
+
+	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
 static DBusMessage *request_scan(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
-	enum connman_device_type type;
+	enum connman_service_type type;
 	const char *str;
 	int err;
 
@@ -365,11 +380,11 @@ static DBusMessage *request_scan(DBusConnection *conn,
 							DBUS_TYPE_INVALID);
 
 	if (g_strcmp0(str, "") == 0)
-		type = CONNMAN_DEVICE_TYPE_UNKNOWN;
+		type = CONNMAN_SERVICE_TYPE_UNKNOWN;
 	else if (g_strcmp0(str, "wifi") == 0)
-		type = CONNMAN_DEVICE_TYPE_WIFI;
+		type = CONNMAN_SERVICE_TYPE_WIFI;
 	else if (g_strcmp0(str, "wimax") == 0)
-		type = CONNMAN_DEVICE_TYPE_WIMAX;
+		type = CONNMAN_SERVICE_TYPE_WIMAX;
 	else
 		return __connman_error_invalid_arguments(msg);
 
@@ -386,80 +401,155 @@ static DBusMessage *request_scan(DBusConnection *conn,
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
+static DBusConnection *connection = NULL;
+
+static enum connman_service_type technology_type;
+static connman_bool_t technology_enabled;
+static DBusMessage *technology_pending = NULL;
+static guint technology_timeout = 0;
+
+static void technology_reply(int error)
+{
+	DBG("");
+
+	if (technology_timeout > 0) {
+		g_source_remove(technology_timeout);
+		technology_timeout = 0;
+	}
+
+	if (technology_pending != NULL) {
+		if (error > 0) {
+			DBusMessage *reply;
+
+			reply = __connman_error_failed(technology_pending,
+								error);
+			if (reply != NULL)
+				g_dbus_send_message(connection, reply);
+		} else
+			g_dbus_send_reply(connection, technology_pending,
+							DBUS_TYPE_INVALID);
+
+		dbus_message_unref(technology_pending);
+		technology_pending = NULL;
+	}
+
+	technology_type = CONNMAN_SERVICE_TYPE_UNKNOWN;
+}
+
+static gboolean technology_abort(gpointer user_data)
+{
+	DBG("");
+
+	technology_timeout = 0;
+
+	technology_reply(ETIMEDOUT);
+
+	return FALSE;
+}
+
+static void technology_notify(enum connman_service_type type,
+						connman_bool_t enabled)
+{
+	DBG("type %d enabled %d", type, enabled);
+
+	if (type == technology_type && enabled == technology_enabled)
+		technology_reply(0);
+}
+
+static struct connman_notifier technology_notifier = {
+	.name		= "manager",
+	.priority	= CONNMAN_NOTIFIER_PRIORITY_HIGH,
+	.service_enabled= technology_notify,
+};
+
 static DBusMessage *enable_technology(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
-	enum connman_device_type type;
+	enum connman_service_type type;
 	const char *str;
 	int err;
 
 	DBG("conn %p", conn);
 
+	if (technology_pending != NULL)
+		return __connman_error_in_progress(msg);
+
 	dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &str,
 							DBUS_TYPE_INVALID);
 
 	if (g_strcmp0(str, "ethernet") == 0)
-		type = CONNMAN_DEVICE_TYPE_ETHERNET;
+		type = CONNMAN_SERVICE_TYPE_ETHERNET;
 	else if (g_strcmp0(str, "wifi") == 0)
-		type = CONNMAN_DEVICE_TYPE_WIFI;
+		type = CONNMAN_SERVICE_TYPE_WIFI;
 	else if (g_strcmp0(str, "wimax") == 0)
-		type = CONNMAN_DEVICE_TYPE_WIMAX;
+		type = CONNMAN_SERVICE_TYPE_WIMAX;
 	else if (g_strcmp0(str, "bluetooth") == 0)
-		type = CONNMAN_DEVICE_TYPE_BLUETOOTH;
-	else if (g_strcmp0(str, "gps") == 0)
-		type = CONNMAN_DEVICE_TYPE_GPS;
+		type = CONNMAN_SERVICE_TYPE_BLUETOOTH;
+	else if (g_strcmp0(str, "cellular") == 0)
+		type = CONNMAN_SERVICE_TYPE_CELLULAR;
 	else
 		return __connman_error_invalid_arguments(msg);
 
+	if (__connman_notifier_is_enabled(type) == TRUE)
+		return __connman_error_already_enabled(msg);
+
+	technology_type = type;
+	technology_enabled = TRUE;
+	technology_pending = dbus_message_ref(msg);
+
 	err = __connman_element_enable_technology(type);
-	if (err < 0) {
-		if (err == -EINPROGRESS) {
-			connman_error("Invalid return code from enable");
-			err = -EINVAL;
-		}
+	if (err < 0 && err != -EINPROGRESS)
+		technology_reply(-err);
+	else
+		technology_timeout = g_timeout_add_seconds(15,
+						technology_abort, NULL);
 
-		return __connman_error_failed(msg, -err);
-	}
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+	return NULL;
 }
 
 static DBusMessage *disable_technology(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
-	enum connman_device_type type;
+	enum connman_service_type type;
 	const char *str;
 	int err;
 
 	DBG("conn %p", conn);
 
+	if (technology_pending != NULL)
+		return __connman_error_in_progress(msg);
+
 	dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &str,
 							DBUS_TYPE_INVALID);
 
 	if (g_strcmp0(str, "ethernet") == 0)
-		type = CONNMAN_DEVICE_TYPE_ETHERNET;
+		type = CONNMAN_SERVICE_TYPE_ETHERNET;
 	else if (g_strcmp0(str, "wifi") == 0)
-		type = CONNMAN_DEVICE_TYPE_WIFI;
+		type = CONNMAN_SERVICE_TYPE_WIFI;
 	else if (g_strcmp0(str, "wimax") == 0)
-		type = CONNMAN_DEVICE_TYPE_WIMAX;
+		type = CONNMAN_SERVICE_TYPE_WIMAX;
 	else if (g_strcmp0(str, "bluetooth") == 0)
-		type = CONNMAN_DEVICE_TYPE_BLUETOOTH;
-	else if (g_strcmp0(str, "gps") == 0)
-		type = CONNMAN_DEVICE_TYPE_GPS;
+		type = CONNMAN_SERVICE_TYPE_BLUETOOTH;
+	else if (g_strcmp0(str, "cellular") == 0)
+		type = CONNMAN_SERVICE_TYPE_CELLULAR;
 	else
 		return __connman_error_invalid_arguments(msg);
 
+	if (__connman_notifier_is_enabled(type) == FALSE)
+		return __connman_error_already_disabled(msg);
+
+	technology_type = type;
+	technology_enabled = FALSE;
+	technology_pending = dbus_message_ref(msg);
+
 	err = __connman_element_disable_technology(type);
-	if (err < 0) {
-		if (err == -EINPROGRESS) {
-			connman_error("Invalid return code from disable");
-			err = -EINVAL;
-		}
+	if (err < 0 && err != -EINPROGRESS)
+		technology_reply(-err);
+	else
+		technology_timeout = g_timeout_add_seconds(10,
+						technology_abort, NULL);
 
-		return __connman_error_failed(msg, -err);
-	}
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+	return NULL;
 }
 
 static DBusMessage *connect_service(DBusConnection *conn,
@@ -489,8 +579,8 @@ static DBusMessage *connect_service(DBusConnection *conn,
 static DBusMessage *register_agent(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
-	DBusMessage *reply;
 	const char *sender, *path;
+	int err;
 
 	DBG("conn %p", conn);
 
@@ -499,22 +589,18 @@ static DBusMessage *register_agent(DBusConnection *conn,
 	dbus_message_get_args(msg, NULL, DBUS_TYPE_OBJECT_PATH, &path,
 							DBUS_TYPE_INVALID);
 
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
+	err = __connman_agent_register(sender, path);
+	if (err < 0)
+		return __connman_error_failed(msg, -err);
 
-	dbus_message_append_args(reply, DBUS_TYPE_INVALID);
-
-	__connman_agent_register(sender, path);
-
-	return reply;
+	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
 static DBusMessage *unregister_agent(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
-	DBusMessage *reply;
 	const char *sender, *path;
+	int err;
 
 	DBG("conn %p", conn);
 
@@ -523,26 +609,24 @@ static DBusMessage *unregister_agent(DBusConnection *conn,
 	dbus_message_get_args(msg, NULL, DBUS_TYPE_OBJECT_PATH, &path,
 							DBUS_TYPE_INVALID);
 
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
+	err = __connman_agent_unregister(sender, path);
+	if (err < 0)
+		return __connman_error_failed(msg, -err);
 
-	dbus_message_append_args(reply, DBUS_TYPE_INVALID);
-
-	__connman_agent_unregister(sender, path);
-
-	return reply;
+	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
 static GDBusMethodTable manager_methods[] = {
 	{ "GetProperties",     "",      "a{sv}", get_properties     },
 	{ "SetProperty",       "sv",    "",      set_property       },
 	{ "GetState",          "",      "s",     get_state          },
-	{ "AddProfile",        "s",     "o",     add_profile        },
+	{ "CreateProfile",     "s",     "o",     create_profile     },
 	{ "RemoveProfile",     "o",     "",      remove_profile     },
 	{ "RequestScan",       "s",     "",      request_scan       },
-	{ "EnableTechnology",  "s",     "",      enable_technology  },
-	{ "DisableTechnology", "s",     "",      disable_technology },
+	{ "EnableTechnology",  "s",     "",      enable_technology,
+						G_DBUS_METHOD_FLAG_ASYNC },
+	{ "DisableTechnology", "s",     "",      disable_technology,
+						G_DBUS_METHOD_FLAG_ASYNC },
 	{ "ConnectService",    "a{sv}", "o",     connect_service,
 						G_DBUS_METHOD_FLAG_ASYNC },
 	{ "RegisterAgent",     "o",     "",      register_agent     },
@@ -608,7 +692,10 @@ static DBusMessage *nm_state(DBusConnection *conn,
 	if (reply == NULL)
 		return NULL;
 
-	state = NM_STATE_DISCONNECTED;
+	if (__connman_element_count(NULL, CONNMAN_ELEMENT_TYPE_CONNECTION) > 0)
+		state = NM_STATE_CONNECTED;
+	else
+		state = NM_STATE_DISCONNECTED;
 
 	dbus_message_append_args(reply, DBUS_TYPE_UINT32, &state,
 							DBUS_TYPE_INVALID);
@@ -623,70 +710,18 @@ static GDBusMethodTable nm_methods[] = {
 	{ },
 };
 
-static int manager_load(void)
-{
-	GKeyFile *keyfile;
-	GError *error = NULL;
-	connman_bool_t offlinemode;
-
-	DBG("");
-
-	keyfile = __connman_storage_open();
-	if (keyfile == NULL)
-		return -EIO;
-
-	offlinemode = g_key_file_get_boolean(keyfile, "global",
-						"OfflineMode", &error);
-	if (error == NULL) {
-		global_offlinemode = offlinemode;
-
-		__connman_device_set_offlinemode(offlinemode);
-	}
-	g_clear_error(&error);
-
-	__connman_storage_close(keyfile, FALSE);
-
-	return 0;
-}
-
-static int manager_save(void)
-{
-	GKeyFile *keyfile;
-
-	DBG("");
-
-	keyfile = __connman_storage_open();
-	if (keyfile == NULL)
-		return -EIO;
-
-	g_key_file_set_boolean(keyfile, "global",
-					"OfflineMode", global_offlinemode);
-
-	__connman_storage_close(keyfile, TRUE);
-
-	return 0;
-}
-
-static struct connman_storage manager_storage = {
-	.name		= "manager",
-	.priority	= CONNMAN_STORAGE_PRIORITY_LOW,
-	.global_load	= manager_load,
-	.global_save	= manager_save,
-};
-
-static DBusConnection *connection = NULL;
 static gboolean nm_compat = FALSE;
 
-int __connman_manager_init(DBusConnection *conn, gboolean compat)
+int __connman_manager_init(gboolean compat)
 {
-	DBG("conn %p", conn);
+	DBG("");
 
-	connection = dbus_connection_ref(conn);
+	connection = connman_dbus_get_connection();
 	if (connection == NULL)
 		return -1;
 
-	if (connman_storage_register(&manager_storage) < 0)
-		connman_error("Failed to register manager storage");
+	if (connman_notifier_register(&technology_notifier) < 0)
+		connman_error("Failed to register technology notifier");
 
 	g_dbus_register_interface(connection, CONNMAN_MANAGER_PATH,
 					CONNMAN_MANAGER_INTERFACE,
@@ -705,9 +740,12 @@ int __connman_manager_init(DBusConnection *conn, gboolean compat)
 
 void __connman_manager_cleanup(void)
 {
-	DBG("conn %p", connection);
+	DBG("");
 
-	connman_storage_unregister(&manager_storage);
+	connman_notifier_unregister(&technology_notifier);
+
+	if (connection == NULL)
+		return;
 
 	if (nm_compat == TRUE) {
 		g_dbus_unregister_interface(connection, NM_PATH, NM_INTERFACE);
